@@ -73,7 +73,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 			if(u->subtype() == SUBTYPE_ALIAS_UNIT && ((AliasUnit*) u)->firstBaseExponent() == 1 && ((AliasUnit*) u)->firstBaseUnit()->subtype() != SUBTYPE_COMPOSITE_UNIT) {
 				size_t idiv = ((AliasUnit*) u)->expression().find("1" DIVISION);
 				if(idiv == 0 && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS, 2) == string::npos) {
-					if((((AliasUnit*) u)->mixWithBase() > 0 || (((AliasUnit*) u)->mixWithBase() == 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (((AliasUnit*) u)->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(((AliasUnit*) u)->mixWithBaseMinimum()))) {
+					if((((AliasUnit*) u)->mixWithBase() > 0 || (((AliasUnit*) u)->mixWithBase() <= 0 && (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER || muc == MIXED_UNITS_CONVERSION_FORCE_ALL))) && (((AliasUnit*) u)->mixWithBaseMinimum() <= 1 || nr.isGreaterThanOrEqualTo(((AliasUnit*) u)->mixWithBaseMinimum()))) {
 						best_u = ((AliasUnit*) u)->firstBaseUnit();
 						MathStructure mstruct_nr(nr);
 						MathStructure m_exp(m_one);
@@ -139,7 +139,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 				mstruct_nr.eval(eo);
 				while(!accept_obsolete && ((AliasUnit*) u)->firstBaseUnit()->subtype() == SUBTYPE_ALIAS_UNIT && abs(((AliasUnit*) ((AliasUnit*) u)->firstBaseUnit())->mixWithBase()) > 1) {
 					u = ((AliasUnit*) u)->firstBaseUnit();
-					if(((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() != 0 || muc == MIXED_UNITS_CONVERSION_FORCE_ALL || (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
+					if(((AliasUnit*) u)->firstBaseExponent() == 1 && (((AliasUnit*) u)->mixWithBase() > 0 || muc == MIXED_UNITS_CONVERSION_FORCE_ALL || (muc == MIXED_UNITS_CONVERSION_FORCE_INTEGER && ((AliasUnit*) u)->expression().find_first_not_of(NUMBERS) == string::npos))) {
 						((AliasUnit*) u)->convertToFirstBaseUnit(mstruct_nr, m_exp);
 						mstruct_nr.eval(eo);
 						if(!mstruct_nr.isNumber() || !m_exp.isOne()) break;
@@ -156,7 +156,7 @@ MathStructure Calculator::convertToMixedUnits(const MathStructure &mstruct, cons
 							non_int = true;
 						}
 					} else {
-						if(((AliasUnit*) u)->mixWithBase() != 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
+						if(((AliasUnit*) u)->mixWithBase() > 0 && muc > MIXED_UNITS_CONVERSION_DEFAULT) muc = MIXED_UNITS_CONVERSION_DEFAULT;
 						u = ((AliasUnit*) u)->firstBaseUnit();
 						nr = mstruct_nr.number();
 						b = true;
@@ -250,9 +250,15 @@ MathStructure Calculator::convertTimeOut(string str, Unit *from_unit, Unit *to_u
 	if(!calculate_thread->write(b_parse)) {calculate_thread->cancel(); return mstruct;}
 	void *x = (void*) &mstruct;
 	if(!calculate_thread->write(x)) {calculate_thread->cancel(); return mstruct;}
-	while(msecs > 0 && b_busy) {
-		sleep_ms(10);
-		msecs -= 10;
+
+	PREPARE_TIMECHECK_VAR
+	if(msecs > 0 && b_busy) {
+		PREPARE_TIMECHECK_TIME(msecs)
+		msecs *= 2;
+		for(int i = 0; b_busy && i < msecs; i++) {
+			sleep_ms(1);
+			DO_TIMECHECK {break;}
+		}
 	}
 	if(had_msecs && b_busy) {
 		abort();
@@ -271,9 +277,11 @@ MathStructure Calculator::convertTimeOut(string str, Unit *from_unit, Unit *to_u
 	if(!calculate_thread->write(b_parse)) {calculate_thread->cancel(); return mstruct;}
 	x = (void*) &mstruct;
 	if(!calculate_thread->write(x)) {calculate_thread->cancel(); return mstruct;}
-	while(msecs > 0 && b_busy) {
-		sleep_ms(10);
-		msecs -= 10;
+	if(msecs > 0 && b_busy) {
+		for(int i = 0; b_busy && i < msecs; i++) {
+			sleep_ms(1);
+			DO_TIMECHECK {break;}
+		}
 	}
 	if(had_msecs && b_busy) {
 		abort();
@@ -1473,7 +1481,7 @@ Unit *Calculator::findMatchingUnit(const MathStructure &mstruct) {
 			if(u != cu && !u->isRegistered()) {
 				if(cu->countUnits() > 1 && u->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 					MathStructure m_u = ((CompositeUnit*) u)->generateMathStructure();
-					if(m_u != cu->generateMathStructure()) {
+					if(m_u != cu->generateMathStructure() && m_u != mstruct) {
 						Unit *u2 = findMatchingUnit(m_u);
 						if(u2) {
 							MathStructure mtest(mstruct);
@@ -1920,21 +1928,25 @@ MathStructure Calculator::convertToOptimalUnit(const MathStructure &mstruct, con
 	eo2.isolate_x = false;
 	eo2.test_comparisons = false;
 	switch(mstruct.type()) {
+		case STRUCT_UNIT: {}
 		case STRUCT_POWER: {
-			if(mstruct.base()->isUnit() && mstruct.exponent()->isNumber() && mstruct.exponent()->number().isRational() && !mstruct.exponent()->number().isZero()) {
-				MathStructure mstruct_new(mstruct);
+			if(mstruct.isUnit() || (mstruct.base()->isUnit() && mstruct.exponent()->isNumber() && mstruct.exponent()->number().isRational() && !mstruct.exponent()->number().isZero())) {
 				int old_points = 0;
 				bool overflow = false;
-				if(mstruct_new.exponent()->isInteger()) old_points = mstruct_new.exponent()->number().intValue(&overflow);
-				else old_points = mstruct_new.exponent()->number().numerator().intValue(&overflow) + mstruct_new.exponent()->number().denominator().intValue() * (mstruct_new.exponent()->number().isNegative() ? -1 : 1);
-				if(overflow) return mstruct_new;
+				Unit *u = (mstruct.isUnit() ? mstruct.unit() : mstruct.base()->unit());
+				if(mstruct.isUnit()) old_points = 1;
+				else if(mstruct.exponent()->isInteger()) old_points = mstruct.exponent()->number().intValue(&overflow);
+				else old_points = mstruct.exponent()->number().numerator().intValue(&overflow) + mstruct.exponent()->number().denominator().intValue() * (mstruct.exponent()->number().isNegative() ? -1 : 1);
+				if(overflow) return mstruct;
+				bool is_si_units = u->isSIUnit();
+				if(old_points == 1 && (is_si_units || !convert_to_si_units) && (!eo.local_currency_conversion || !u->isCurrency())) return mstruct;
 				bool old_minus = false;
 				if(old_points < 0) {
 					old_points = -old_points;
 					old_minus = true;
 				}
-				bool is_si_units = mstruct_new.base()->unit()->isSIUnit();
-				if(mstruct_new.base()->unit()->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
+				MathStructure mstruct_new(mstruct);
+				if(u->baseUnit()->subtype() == SUBTYPE_COMPOSITE_UNIT) {
 					mstruct_new.convertToBaseUnits(true, NULL, true, eo2, true);
 					if(mstruct_new.equals(mstruct, true, true)) {
 						return mstruct_new;
@@ -1944,21 +1956,31 @@ MathStructure Calculator::convertToOptimalUnit(const MathStructure &mstruct, con
 					mstruct_new = convertToOptimalUnit(mstruct_new, eo, convert_to_si_units);
 					if(mstruct_new.equals(mstruct, true, true)) return mstruct_new;
 				} else {
-					CompositeUnit *cu = new CompositeUnit("", "temporary_composite_convert_to_optimal_unit");
-					cu->add(mstruct_new.base()->unit(), mstruct_new.exponent()->number().numerator().intValue());
-					Unit *u = getOptimalUnit(cu, false, eo.local_currency_conversion);
-					if(u == cu) {
+					Unit *u_new = u;
+					if(mstruct_new.isUnit()) {
+						u_new = getOptimalUnit(u, false, eo.local_currency_conversion);
+						if(u_new == u) return mstruct_new;
+						if(eo.approximation == APPROXIMATION_EXACT && u->hasApproximateRelationTo(u_new, true)) {
+							if(!u_new->isRegistered()) delete u_new;
+							return mstruct_new;
+						}
+					} else {
+						CompositeUnit *cu = new CompositeUnit("", "temporary_composite_convert_to_optimal_unit");
+						cu->add(u, mstruct_new.exponent()->number().numerator().intValue());
+						u_new = getOptimalUnit(cu, false, eo.local_currency_conversion);
+						if(u_new == cu) {
+							delete cu;
+							return mstruct_new;
+						}
+						if(eo.approximation == APPROXIMATION_EXACT && cu->hasApproximateRelationTo(u_new, true)) {
+							if(!u_new->isRegistered()) delete u_new;
+							delete cu;
+							return mstruct_new;
+						}
 						delete cu;
-						return mstruct_new;
 					}
-					if(eo.approximation == APPROXIMATION_EXACT && cu->hasApproximateRelationTo(u, true)) {
-						if(!u->isRegistered()) delete u;
-						delete cu;
-						return mstruct_new;
-					}
-					delete cu;
-					mstruct_new = convert(mstruct_new, u, eo, true);
-					if(!u->isRegistered()) delete u;
+					mstruct_new = convert(mstruct_new, u_new, eo, true);
+					if(!u_new->isRegistered()) delete u_new;
 					UNFORMAT(mstruct_new);
 				}
 				int new_points = 0;
@@ -2074,20 +2096,6 @@ MathStructure Calculator::convertToOptimalUnit(const MathStructure &mstruct, con
 			}
 			return mstruct_new;
 		}
-		case STRUCT_UNIT: {
-			if((!mstruct.unit()->isCurrency() || !eo.local_currency_conversion) && (!convert_to_si_units || mstruct.unit()->isSIUnit())) return mstruct;
-			Unit *u = getOptimalUnit(mstruct.unit(), false, eo.local_currency_conversion);
-			if(u != mstruct.unit()) {
-				if((u->isSIUnit() || (u->isCurrency() && eo.local_currency_conversion)) && (eo.approximation != APPROXIMATION_EXACT || !mstruct.unit()->hasApproximateRelationTo(u, true))) {
-					MathStructure mstruct_new = convert(mstruct, u, eo, true);
-					if(!u->isRegistered()) delete u;
-					UNFORMAT(mstruct_new)
-					return mstruct_new;
-				}
-				if(!u->isRegistered()) delete u;
-			}
-			break;
-		}
 		case STRUCT_MULTIPLICATION: {
 			if(!mstruct.containsType(STRUCT_UNIT, true)) return mstruct;
 			int old_points = 0;
@@ -2295,6 +2303,7 @@ MathStructure Calculator::convert(const MathStructure &mstruct_to_convert, strin
 			if(v->referenceName() == "bohr_radius") u = CALCULATOR->getActiveUnit("bohr_unit");
 			else if(v->referenceName() == "elementary_charge") u = CALCULATOR->getActiveUnit("e_unit");
 			else if(v->referenceName() == "electron_mass") u = CALCULATOR->getActiveUnit("electron_unit");
+			else if(v->referenceName() == "compton_wavelength_2pi") u = CALCULATOR->getActiveUnit("l_N");
 		}
 		if(u) v = NULL;
 	}
